#!/usr/bin/perl
#
# cdencode.perl
#
# this controls the ripping of music from a CD and encoding it
# into the database.
#

$tunebot_root=$ENV{TUNEBOT_ROOT};
$ses_directory=$tunebot_root."/data/loader";
$framesec=75;

if ($#ARGV != 1) {
    exit 1;
}
$usrnam=$ARGV[0];
$sesno=$ARGV[1];

sub readTunebotSettings {
    my ($i, $j);
    open (I, "$tunebot_root/settings");
    while ($i = <I>) {
	chomp $i;
	$j = index($i, "=");
	if ($j > 0) {
	    $settings{substr($i,0,$j)} = substr($i,$j+1);
	}
    }
    close(I);
}

readTunebotSettings;

$cdpara=$settings{"CDPARANOIA_PATH"};
$blade=$settings{"BLADEENC_PATH"};
$id3tool=$settings{"ID3TOOL_PATH"};

#
# add/replace a tag value into the current session.
# assumes $ses_directory, $usrnam and $sesno are set.
# creates the session if it doesn't already exist.
#
sub addSesTag {
    open (O, ">>$ses_directory/$usrnam.$sesno");
    print O @_, "\n";
    close(O);
    chmod 0666, "$ses_directory/$usrnam.$sesno";
}

#
# lookup a tag value from the current session.
# assumes $ses_directory, $usrnam and $sesno are set.
# returns the tag value, or "" if not defined.
#
sub getSesTag {
    my ($tag) = $_[0] . "=";
    my ($l) = length($tag);
    my $val;
    my $i;
    open (I, "$ses_directory/$usrnam.$sesno");
    while ($i = <I>) {
	chomp $i;
	if (substr($i, 0, $l) eq $tag) {
	    $val = $i;
	    $val =~ s/^[^=]*=//;
	}
    }
    close(I);
    return $val;
}

#
# lookup the entire history of a tag value from the current session.
# assumes $ses_directory, $usrnam and $sesno are set.
# returns an array of values, in order found.
#
sub getSesTagAll {
    my ($tag) = $_[0] . "=";
    my ($l) = length($tag);
    my $val;
    my @lst;
    my $i;
    open (I, "$ses_directory/$usrnam.$sesno");
    while ($i = <I>) {
	chomp $i;
	if (substr($i, 0, $l) eq $tag) {
	    $val = $i;
	    $val =~ s/^[^=]*=//;
	    @lst = (@lst, $val);
	}
    }
    close(I);
    return @lst;
}

addSesTag("STATE=ENCODING");

#addSesTag("MESSAGE=before sleep");
#sleep 2;
#addSesTag("MESSAGE=after sleep");

#
# figure out how much space we need for the MP3 files
#
$mp3rate = getSesTag("MP3RATE");
if ($mp3rate != 128 && $mp3rate != 192 && $mp3rate != 256) {
    addSesTag("MESSAGE=MP3 Rate $mp3rate not supported");
    addSesTag("STATE=IDLE");
    exit 0;
}
$sneed = getSesTag("TOTLENGTH");
if ($sneed < 1) {
    addSesTag("MESSAGE=The total length of the CD doesn't seem right");
    addSesTag("STATE=IDLE");
    exit 0;
}
$sneed = ($sneed * ($mp3rate * 1024 / 8)) / 1048576;
$sneed = int($sneed + 1.5);

$ntracks = getSesTag("NTRACKS");
if ($ntracks < 1) {
    addSesTag("MESSAGE=The number of tracks is zero?");
    addSesTag("STATE=IDLE");
    exit 0;
}

#
# figure out how much temp space the wav files will need.
# this is a slightly conservative estimate.
#
# for "SMALL" encoding plan, figure out temp space for biggest track:
#    track length / number of frames per second
#     * samples per second * size of sample
# for "QUICK", temp space is sum of all tracks.
#
@tl = split /,/, getSesTag("LENGTHS");
if ($#tl < $ntracks - 1) {
    addSesTag("MESSAGE=Don't seem to have tracks length information?");
    addSesTag("STATE=IDLE");
    exit 0;
}
$stemp = 0;
$plan = getSesTag("ENCODINGPLAN");
for ($i = 0; $i < $ntracks; ++$i) {
    $s = ($tl[$i] / $framesec) * 44100 * 4;
    if ($plan eq "QUICK") {
	$stemp += $s;
    } else {
	if ($s > $stemp) {
	    $stemp = $s;
	}
    }
}
$stemp = int($stemp / 1048576 + 1.5);
addSesTag("MESSAGE=Need $sneed megabytes plus $stemp temporary");
$sneed = $sneed + $stemp;

#
# find a partition under music that has enough room
#
$targetdir = "";
@musicdirs = split /:/, $settings{"MUSIC"};
foreach $d (@musicdirs) {
    $s = 0;
    open(I, "df $d |");
    while (<I>) {
	if (/\//) {
	    chomp;
	    split;
	    # If the remote filesystem string is too long, df introduces
	    # a newline after the first column of output. We need to ignore
	    # this line, which has only one column, and we need to compensate
	    # for the fact that the following line has one less column. This
	    # is probably a bug in Linux df, as I note that Solaris df only
	    # dies this if stdout is a tty. Anyway, we work around it.
	    $ncol = scalar @_;
	    next if $ncol == 1;
	    $s = $_[($ncol == 6) ? 3 : 2] / 1024;
	}
    }
    close(I);
    if ($s > $sneed) {
	addSesTag("MESSAGE=Partition $d has sufficient space");
	if ($targetdir eq "") {
	    $targetdir = $d;
	}
    }
}
if ($targetdir eq "") {
    addSesTag("MESSAGE=Can't find a music partition with enough space");
    addSesTag("STATE=IDLE");
    exit 0;
}

$artistdir = getSesTag("ARTISTDIRECTORY");
$artistname = getSesTag("ARTISTNAME");
$artistsort = getSesTag("ARTISTSORT");
$albumdir = getSesTag("ALBUMDIRECTORY");
$albumname = getSesTag("ALBUMNAME");
if ($artistdir eq "" || $artistname eq "" || $albumdir eq "") {
    addSesTag("MESSAGE=An artist or album directory or name is null");
    addSesTag("STATE=IDLE");
    exit 0;
}

if (-d "$targetdir/$artistdir") {
    addSesTag("MESSAGE=Artist directory $targetdir/$artistdir already exists, reusing");
} else {
    addSesTag("MESSAGE=Creating Artist directory $targetdir/$artistdir");
    mkdir "$targetdir/$artistdir", 0775;
    chmod 0775, "$targetdir/$artistdir";
    open(O, ">$targetdir/$artistdir.t");
    print O "N=$artistname\n";
    if ($artistsort ne "") {
	print O "SORT=$artistsort\n";
    }
    close(O);
    chmod 0664, "$targetdir/$artistdir.t";
}

if (-d "$targetdir/$artistdir/$albumdir") {
    addSesTag("MESSAGE=Album directory $targetdir/$artistdir/$albumdir already exists, stopping");
    addSesTag("STATE=IDLE");
    exit 0;
}

addSesTag("MESSAGE=Creating Album directory $targetdir/$artistdir/$albumdir");
mkdir "$targetdir/$artistdir/$albumdir", 0775;
chmod 0775, "$targetdir/$artistdir/$albumdir";
$aifile=getSesTag("AIFILE");
if ($aifile ne "") {
    addSesTag("MESSAGE=Installing cover art");
    system("/bin/cp $ses_directory/$aifile $targetdir/$artistdir/$albumdir/coverart.jpg");
}
open(O, ">$targetdir/$artistdir/$albumdir.t");
print O "N=$albumname\n";
if (getSesTag("CDDB") ne "") {
    print O "CDDB=".getSesTag("CDDB")."\n";
}
if (getSesTag("ALBUMDATE") ne "") {
    print O "DATE=".getSesTag("ALBUMDATE")."\n";
}
if ($aifile ne "") {
    print O "ICON=coverart.jpg\n";
}
close(O);
chmod 0664, "$targetdir/$artistdir/$albumdir.t";

if (! chdir("$targetdir/$artistdir/$albumdir")) {
    addSesTag("MESSAGE=Can't change to target directory");
    addSesTag("STATE=IDLE");
    exit 0;
}

#
# XXX put these in temp file names until all done.
# XXX that may improve error/restart recovery
#

if ($plan eq "QUICK") {
    #
    # the "QUICK" ripping strategy - get the disc out quickly.
    #
    for ($i = 1; $i <= $ntracks; ++$i) {
	$fbase = sprintf("track%02d", $i);
	addSesTag("MESSAGE=Ripping track $i of $ntracks");
	system "$cdpara $i $fbase.wav </dev/null >/dev/null 2>/dev/null";
    }
    addSesTag("MESSAGE=I am done with the disc");
    for ($i = 1; $i <= $ntracks; ++$i) {
	$fbase = sprintf("track%02d", $i);
	addSesTag("MESSAGE=Encoding track $i of $ntracks");
	system "$blade -QUIT -$mp3rate $fbase.wav </dev/null >/dev/null 2>/dev/null";
	unlink "$fbase.wav";
    }


} else {
    #
    # the "SMALL" ripping strategy - encode each track as we rip.
    #
    for ($i = 1; $i <= $ntracks; ++$i) {
#	print "$i\n";
	$fbase = sprintf("track%02d", $i);

	addSesTag("MESSAGE=Ripping track $i of $ntracks");
#	system "$cdpara $i-$i:[2] $fbase.wav </dev/null >/dev/null 2>/dev/null";
	system "$cdpara $i $fbase.wav </dev/null >/dev/null 2>/dev/null";
	addSesTag("MESSAGE=Encoding track $i of $ntracks");
	system "$blade -QUIT -$mp3rate $fbase.wav </dev/null >/dev/null 2>/dev/null";
	unlink "$fbase.wav";
    }
}

#
# add MP3 ids and generate tag files
#
for ($i = 1; $i <= $ntracks; ++$i) {
    $fbase = sprintf("track%02d", $i);
    if ($id3tool ne "") {
	@idcmd=($id3tool);
	@idcmd=(@idcmd, "--set-title=".getSesTag("TRACKNAME$i"));
	@idcmd=(@idcmd, "--set-album=$albumname");
	if (getSesTag("TRACKARTIST$i") ne "") {
	    @idcmd=(@idcmd, "--set-artist=".getSesTag("TRACKARTIST$i"));
	} else {
	    @idcmd=(@idcmd, "--set-artist=$artistname");
	}
	if (getSesTag("ALBUMDATE") ne "") {
	    @idcmd=(@idcmd, "--set-year=".getSesTag("ALBUMDATE"));
	}
	@idcmd=(@idcmd, "$fbase.mp3");
	system @idcmd;
    }
    chmod 0664, "$fbase.mp3";

    open(O, ">$fbase.t");
    print O "N=".getSesTag("TRACKNAME$i")."\n";
    print O "T=".getSesTag("TRACKTIME$i")."\n";
    if (getSesTag("TRACKARTIST$i") ne "") {
	print O "ARTIST=".getSesTag("TRACKARTIST$i")."\n";
    }
    close(O);
    chmod 0664, "$fbase.t";
}

addSesTag("MESSAGE=Finished encoding CD");

#
# XXX here we would move all the files to their real names
#
addSesTag("MESSAGE=Grinding indexes");
system "$tunebot_root/bin/grind-indexes $targetdir </dev/null >/dev/null 2>/dev/null";
addSesTag("MESSAGE=Finished grinding indexes");

addSesTag("STATE=IDLE");

exit 0;

