#!/usr/bin/perl
#
# tunectl.cgi
#
# display/edit tunebot global controls.
#

########################################################################
# perl script boilerplate.
# set tunebot root and load utility module.
#
BEGIN {
    $tunebot_root="XXX-the-tunebot-install-root";
    @INC=($tunebot_root."/lib", @INC);
}
use Tunebot;
#
# end of perl boilerplate
########################################################################

## setcontrol(name, value)
#
# rewrite the given line in the controls file, or add it if it didn't exist.
#
sub setcontrol {
    my $n=$_[0];
    my $v=$_[1];
    my @c=();
    my $line;
    my $unf=1;
    open(I, $controls);
    while ($line = <I>) {
	chomp $line;
	if ($line =~ /^$n=/) {
	    @c=(@c, "$n=$v");
	    $unf=0;
	} else {
	    @c=(@c, $line);
	}
    }
    close(I);
    if ($unf) {
	@c=(@c, "$n=$v");
    }
    open(O, ">$controls");
    foreach $line (@c) {
	print O "$line\n";
    }
    close(O);
}

$cgibinurl=$configs{"CGI_BIN_URL"};
$topurl=$configs{"HTML_TOP_URL"};

$controls=$configs{"CONTROLS"};

Tunebot::suckInCookies;
$tunebot_user=$cookie{TUNEBOTUSER};

Tunebot::suckInPostGet;

Tunebot::suckInTags();
Tunebot::suckInAutotags();

#
# this is the list of controls that can be modified, in order:
#  - variable name
#  - description
#  - longer explanation
#  - display format (INT or BOOL or a printf string)
#  - list of lists of choices to display and parameter values.
#  - scope of variable: 'g' is global, 'i' is individual - can have either
#    or both.
#
@ctrls = (
    {
	name => "RANDOM_IF_IDLE",
	desc => "No dead air",
	expl => "If this is set to Yes, tunebot will continuously play"
		." random stuff when nothing is requested.  If something"
		." is requested, it gets played first (except that there"
		." can be a probability of playing something random first).",
	scope => "g",
	fmt => "BOOL",
	choices => [
	    [ "Yes", "1" ],
	    [ "No", "0" ],
	]
    },
    {
	name => "RANDOM_FROM_Q",
	desc => "Play requests in random order",
	expl => "Is this is set to Yes, tracks are picked randomly from"
		." the requests.  Otherwise, requests are played in the"
		." the order they were received.",
	scope => "g",
	fmt => "BOOL",
	choices => [
	    [ "Yes", "1" ],
	    [ "No", "0" ],
	]
    },
    {
	name => "PREEMPT_Q_PERC",
	desc => "Probability of playing a random tune before request",
	expl => "If this is set to zero, no random tunes will be played"
		." as long as there are requests.  Otherwise, it gives"
		." the probability of choosing a random track instead of"
		." pick a request.",
	scope => "g",
	fmt => "%s%%",
	choices => [
	    [ "50%", "50" ],
	    [ "30%", "30" ],
	    [ "20%", "20" ],
	    [ "10%", "10" ],
	    [ "0%", "0" ],
	]
    },
    {
	name => "FILLET_TIME",
	desc => "Announcement interval",
	expl => "If this is set, tunebot will announce the time and play"
		." a short filler track after the number of minutes given.",
	scope => "g",
	fmt => "INT",
	choices => [
	    [ "30", "30" ],
	    [ "45", "45" ],
	    [ "60", "60" ],
	    [ "90", "90" ],
	    [ "120", "120" ],
	    [ "don't", "0" ],
	]
    },
    {
	name => "HOWMANY_TUNES",
	desc => "Default Whuzzon history",
	expl => "The number of recently played tracks to show on the"
		." whuzzon page.",
	scope => "gi",
	fmt => "INT",
	choices => [
	    [ "50", "50" ],
	    [ "20", "20" ],
	    [ "10", "10" ],
	    [ "5", "5" ],
	    [ "2", "2" ],
	    [ "1", "1" ],
	]
    },
    {
	name => "RELOAD_WHUZZON",
	desc => "Auto-reload Whuzzon page",
	expl => "Turning this on makes the Whuzzon page include a meta"
		." tag to reload itself every minute, which kinda keeps"
		." the browser current with what's playing.  This"
		." may eventually cause some browsers to wedge.  I don't"
		." know why that is.",
	scope => "gi",
	fmt => "BOOL",
	choices => [
	    [ "Yes", "60" ],
	    [ "No", "0" ],
	]
    },
    {
	name => "NOW_PLAYING_COVER",
	desc => "Show album cover in Whuzzon",
	expl => "If set to yes, a small image of the album cover for the"
		." current track is shown on the whuzzon page.",
	scope => "gi",
	fmt => "BOOL",
	choices => [
	    [ "Yes", "1" ],
	    [ "No", "0" ],
	]
    },
    {
	name => "PUT_BACK_INDEX",
	desc => "Jump back to index on request",
	expl => "Set this to Yes to have tunebot jump back to the"
		." alpha index after taking a request (normally"
		." where you just came from, but not always)."
		."  Otherwise it will go to the Whuzzon page.",
	scope => "gi",
	fmt => "BOOL",
	choices => [
	    [ "Yes", "1" ],
	    [ "No", "0" ],
	]
    },
    {
	name => "INDEX_ALBUMS",
	desc => "Number of albums to show per index page",
	scope => "gi",
	fmt => "INT",
	choices => [
	    [ "100", "100" ],
	    [ "50", "50" ],
	    [ "20", "20" ],
	    [ "10", "10" ],
	    [ "5", "5" ],
	    [ "2", "2" ],
	    [ "1", "1" ],
	]
    },
    {
	name => "AVOID_ANCHORS",
	desc => "Avoid use of page anchors",
	expl => "Set this to avoid using page anchors in web pages."
		."  Page anchors cause some browsers to act erratically.",
	scope => "gi",
	fmt => "BOOL",
	choices => [
	    [ "Yes", "1" ],
	    [ "No", "0" ],
	]
    },
    {
	name => "PLAY_CONTROLS",
	desc => "Interactive player controls",
	expl => "Turn this on to display little controls on the whuzzon"
		." page (and in other page headers, if whuzzon is enabled"
		." everywhere) that let you replay, cancel or pause"
		." the player.",
	scope => "g",
	fmt => "BOOL",
	choices => [
	    [ "Yes", "1" ],
	    [ "No", "0" ],
	]
    },
    {
	name => "WHUZZON_IN_HEADER",
	desc => "Show whuzzon in all page headers",
	expl => "This makes the currently playing track title and artist"
		." info show up in all page headers (and the skip/replay"
		." controls, if enabled).",
	scope => "gi",
	fmt => "BOOL",
	choices => [
	    [ "Yes", "1" ],
	    [ "No", "0" ],
	]
    },
    {
	name => "HIGHLIGHT_RECENT",
	desc => "Highlight recently played tracks",
	expl => "This makes tracks that were played recently show up bold"
		." in the index pages so they're easier to spot.",
	scope => "gi",
	fmt => "BOOL",
	choices => [
	    [ "Yes", "1" ],
	    [ "No", "0" ],
	]
    },
    {
	name => "HOW_RECENT",
	desc => "How many tracks are 'recent'",
	expl => "This is the number of tracks to consider 'recently played'.",
	scope => "gi",
	fmt => "INT",
	choices => [
	    [ "300", "300" ],
	    [ "100", "100" ],
	    [ "30", "30" ],
	    [ "10", "10" ],
	]
    },
    {
	name => "FLAP_PAGES",
	desc => "Run pageflap if mp3 player fails",
	expl => "If this is set to Yes, tunebot runs the pageflap"
		." program if the mp3 player exits with nonzero status,"
		." then replays the track.  This used to be needed due"
		." to a bug in the 2.0 (2.2?) kernels.  The kernel"
		." memory would get fragmented, the kernel would"
		." give away low memory and the audio driver would"
		." be unable to allocate a DMA buffer.  Mallocing and"
		." releasing all the memory in the system would cause"
		." a cleanup.",
	scope => "g",
	fmt => "BOOL",
	choices => [
	    [ "Yes", "1" ],
	    [ "No", "0" ],
	]
    },
    {
	name => "MYSTERY_PARAM",
	desc => "Some mystery parameter",
	expl => "This is the explanation of the mystery parameter.",
	scope => "g",
	fmt => "BOOL",
	choices => [
	    [ "Yes", "1" ],
	    [ "No", "0" ],
	]
    },
);

#
# turn the controls list into a hash so we can check names
#
foreach $x (@ctrls) {
    $ctrl{$x->{name}}=$x;
}

$errmsg="";
#
# set control if we got a posted name and new value
# we can't print errors yet...
#
if ($posts{name} ne "" && $posts{val} ne "") {
    if ($ctrl{$posts{name}} ne "") {
	$configs{$posts{name}} = $posts{val};
	setcontrol($posts{name}, $posts{val});
    } else {
	$errmsg="Unable to set control <em>$posts{name}</em>, sorry.\n";
    }
}

#
# start displaying the page
# print any error from before.
#
Tunebot::printPageHeader "Control Panel";
print "<center>\n";
print "<font size=+2><b>Control Panel</b></font>\n";
print "<br>Tunebot version ".$configs{TUNEBOT_VERSION}."\n";
print "</center>\n";

print $errmsg;

#
# render table of current (new) parameter values
#
print "<p>\n";
print "<table width=85% align=center>\n";
print "<tr> <td width=6%> </td> <td width=56%> </td></tr>\n";
print "<tr><th colspan=2 align=left>Control</th><th>Value</th><th>Modify</th></tr>\n";
print "<tr><td colspan=4><hr></td></tr>\n";

foreach $x (@ctrls) {
    $j=$x->{desc};
    if ($x->{expl} ne "") {
	if ($posts{explain} eq $x->{name}) {
	    $j="<a href=\"$cgibinurl/tunectl.cgi\">$j</a>";
	} else {
	    $j="<a href=\"$cgibinurl/tunectl.cgi?explain=".$x->{name}."\">$j</a>";
	}
    }
    print "<tr><td colspan=2>$j</td>\n";
    $fmt=$x->{fmt};
    print " <td align=center>";
#    $configs{$x->{name}};
    if ($configs{$x->{name}} eq "") {
	$configs{$x->{name}} = "0";
    }
    if ($fmt eq "INT") {
	print $configs{$x->{name}};
    } elsif ($fmt eq "BOOL") {
	if ($configs{$x->{name}} != 0) {
	    print "Yes";
	} else {
	    print "No";
	}
    } else {
	printf $x->{fmt}, $configs{$x->{name}};
    }
    print "</td>\n";
    print " <td align=center>\n";
    $j="";
    foreach $c (@{$x->{choices}}) {
	print "  $j<a href=\"$cgibinurl/tunectl.cgi?name=".$x->{name}."&val=".$c->[1]."\">".$c->[0]."</a>\n";
	$j=" : ";
    }
    print " </td>\n";
    print "</tr>\n";
    if ($posts{explain} eq $x->{name}) {
	print "<tr><td></td><td><font color=white><em>"
		.$x->{expl}."</em></font></td></tr>\n";
    }
}


print "<tr><td colspan=4><hr></td></tr>\n";
print "</table>\n";

Tunebot::printPageTrailer;
exit 0

