#
# Tunebot.pm
#
# 2001-6-2 manchek
#
# Tunebot perl module.
# Does all the tasty common stuff that tunebot programs need.
#

package Tunebot;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(%configs %tags %autotags %posts %cookie);
@EXPORT_OK = qw(%configs %tags %autotags %posts %cookie);

##################
#  startup code  #
#                #
#################

#
# read configuration file into %configs hash
# the config file is in (limiter) shell syntax, with lines of form:
#    VARIABLE=value.
# read in each line and stick into %configs, with the variable name
# as key.  expand quotes ["'] and backslashes.  expand $variable
# syntax using already-defined configs.
#

# start with TUNEBOT_ROOT defined, in case it's not set in file.
$configs{"TUNEBOT_ROOT"}=$main::tunebot_root;

sub parseSettings {
    while ($line = <I>) {
	if ($line =~ /^[A-Za-z0-9_]+=/) {
	    chomp $line;
	    ($var = $line) =~ s/=.*//;
	    ($val = $line) =~ s/^[^=]*=//;
	    $val2 = "";
	    $qm="";
	    while ($val ne "") {
		$c = substr($val, 0, 1);
		$val = substr($val, 1);
		if ($qm ne "") {
		    if ($qm eq "\$") {
			if ($qm eq $c) {
			    $val2 .= "\$";
			} else {
			    $val=$c . $val;
			    ($c = $val) =~ s/[^A-Za-z0-9_].*//;
			    $val =~ s/^[A-Za-z0-9_]*//;
			    $val2 .= $configs{$c};
			}
			$qm="";
		    } elsif ($qm eq "\\") {
			$val2 .= $c;
			$qm="";
		    } elsif ($qm eq $c) {
			$qm="";
		    } else {
			$val2 .= $c;
		    }
		} else {
		    if ($c eq "\"" || $c eq "'" || $c eq "\\" || $c eq "\$") {
			$qm=$c;
		    } else {
			$val2 .= $c;
		    }
		}
	    }
#	    print $var, " = \"", $val2 , "\"\n";
	    $configs{$var}=$val2;
	}
    }
}

open(I, $main::tunebot_root . "/settings");
parseSettings;
close(I);
open(I, $main::tunebot_root . "/controls");
parseSettings;
close(I);

$cgibinurl=$configs{"CGI_BIN_URL"};

###########################
#  url library functions  #
#                         #
##########################

## urldecode(url_encoded_string)
#
# convert a URL-encoded string to a regular string:
#  translate '+' to ' '.
#  expand characters encoded as 2 hex digits (%00).
#
sub urldecode {
    my ($a) = $_[0];
    my ($b) = "";
    $a =~ tr/+/ /;
    while ($a ne "") {
        my ($c) = substr($a, 0, 1);
        if ($c eq "%") {
            $b .= chr hex substr($a, 1, 2);
            $a =~ s/...//;
        } else {
            $b .= $c;
            $a =~ s/.//;
        }
    }
    return $b;
}

## urlencode(string)
#
# encode a regular string in URL syntax:
#  translate unsafe characters using '+' for space and %xx
#  notation for others.
#
sub urlencode {
    my ($a)=$_[0];
    $a =~ s/&/&amp;/g;
    $a =~ s/</&lt;/g;
    $a =~ s/>/&gt;/g;
    $a =~ s/"/&quot;/g;
    return $a;
}
#sub urlencode {
#    my ($a)=$_[0];
#    my ($b)="";
#    while ($a ne "") {
#        my ($c) = substr($a, 0, 1);
#	if ($c =~ /[a-zA-Z0-9]/) {
#            $b .= $c;
#	} else {
#            $b .= sprintf("%%%02x",ord($c));
#	}
#	$a =~ s/.//;
#    }
#    return $b;
#}

## suckInPost()
#
# parse stdin into a list of name-value pairs and stuff into hash %posts.
# this is used to support the POST http method with default encoding type
# application/x-www-form-urlencoded.
# stdin format is "name1=value1&name2=value2&...".
#
sub suckInPost {
    while ($line = <>) {
	chomp $line;
	@nvpairs = split('&', $line);
	for ($i = 0; $i <= $#nvpairs; $i++) {
	    @nv = split('=',@nvpairs[$i]);
	    if ($#nv == 1) {
		if ($debug > 0) {
		    print @nv[0] . "  =  " . @nv[1] . "\n";
		}
		$posts{@nv[0]} = urldecode(@nv[1]);
	    }
	}
    };
}

## suckInGet()
#
# parse environment QUERY_STRING into nv pairs and hash them.
# this supports the GET http method.
#
sub suckInGet {
    @nvpairs = split('&', $ENV{QUERY_STRING});
    for ($i = 0; $i <= $#nvpairs; $i++) {
	@nv = split('=',@nvpairs[$i]);
	if ($#nv == 1) {
	    if ($debug > 0) {
		print @nv[0] . "  =  " . @nv[1] . "\n";
	    }
	    $posts{@nv[0]} = urldecode(@nv[1]);
	}
    }
}

## suckInPostGet()
#
# suck in stdin and environment QUERY_STRING, to support either
# POST or GET http methods.
#
sub suckInPostGet {
    if ($debug > 0) {
	print "<p>\n";
	print "query string is: " . $ENV{QUERY_STRING} . "\n";
	print "<p>\n";
	print "i've got " . $#ARGV . " command args\n";
	for ($i = 0; $i <= $#ARGV; $i++) {
	    print "<br>" . $i . ". " . @ARGV[$i] . "\n";
	}
	print "<p>\n";
	print "<pre>\n";
    }
    suckInPost;
    suckInGet;
    if ($debug > 0) {
	print "</pre>\n";
    }
}

## suckInCookies()
#
# suck in cookies from HTTP_COOKIE envar and put them in %cookie.
#
sub suckInCookies {
    my $clist=$ENV{HTTP_COOKIE};
    my ($c,$n,$v);
    while ($clist =~ /=/) {
	($c=$clist) =~ s/;.*//;
	$clist = substr($clist, length($c));
	$clist =~ s/^; *//;
	($n=$c) =~ s/=.*//;
	($v=$c) =~ s/[^=]*=//;
	$cookie{$n}=$v;
    }
}

#############################
#  tags database functions  #
#                           #
############################

## suckInTags()
#
# make %tags hash available.
# key of hash is { file_path, tag_name }.
#
sub suckInTags {
    use DB_File;
    tie %tags, "DB_File", $main::tunebot_root."/tune.tags.db", O_RDONLY, 0644;
}

## suckInAutotags()
#
# make %autotags hash available.
# key of hash is { file_path, tag_name }.
#
sub suckInAutotags {
    use DB_File;
    tie %autotags, "DB_File", $main::tunebot_root."/tune.autotags.db", O_RDONLY, 0644;
}

## suckInTagsFromFile()
#
# read all track tags into %tags hash.
# key of hash is { file_path, tag_name }.
#
sub suckInTagsFromFile {
    open(TUNETAGS, $configs{TUNETAGS});
    while ($line = <TUNETAGS>) {
	chop $line;
	$i = index($line, ":");
	$j = index($line, "=", $i);
	$file = substr($line, 0, $i);
	$tag = substr($line, $i + 1, $j - $i - 1);
	$value = substr($line, $j + 1);
	if ($tag eq "SORT") {
	    $value =~ tr/a-z/A-Z/;
	}
	$tags{$file, $tag} = $value;
    }
    close(TUNETAGS);
}

## suckInAutotagsFromFile()
#
# read all track auto-generated tags into %autotags hash.
# key of hash is { file_path, tag_name }.
#
sub suckInAutotagsFromFile {
    open(AUTOTAGS, $main::tunebot_root."/tune.autotags");
    while ($line = <AUTOTAGS>) {
	chop $line;
	$i = index($line, ":");
	$j = index($line, "=", $i);
	$file = substr($line, 0, $i);
	$tag = substr($line, $i + 1, $j - $i - 1);
	$value = substr($line, $j + 1);
	$autotags{$file, $tag} = $value;
    }
    close(AUTOTAGS);
}

## lookupTagInFile(file,tag)
#
# look up a tag value directly in a file.
# not typically used with track databases, but useful for other files.
# both "file.t" and "file" are tried, in that order.
# tag may be specified as "TAG=" or just "TAG".
#
sub lookupTagInFile {
    my ($f) = @_[0];
    my ($i) = index($f, "\.mp3");
    if ($i >= 0) {
	$f = substr($f, 0, $i);
    }
    my ($t) = @_[1];
    $t =~ s/=$//;
    $t .= "=";
    my ($v) = "";
    if (open(F, $f . ".t") || open(F, $f)) {
	while ($i = <F>) {
	    chop $i;
	    if (substr($i, 0, length($t)) eq $t) {
		$v = substr($i, length($t));
		break;
	    }
	}
	close(F);
    }
    return $v;
}

## rewriteTagInFile(file, tag, value)
#
# set a tag value in a file.
# if the tag already exists in the file, it is replaced,
# otherwise a new tag and value are appended.
# tag name may be specified as "TAG=" or just "TAG".
#
sub rewriteTagInFile {
    my ($f) = @_[0];
    my ($t) = @_[1];
    my ($v) = @_[2];
    my $line;
    my $x = 0;
    $t =~ s/=$//;
    $t .= "=";
    open(I, $f);
    open(O, ">$f.tagtmp");
    chmod 0666, "$f.tagtmp";
    while ($line = <I>) {
	if (substr($line, 0, length($t)) eq $t) {
	    print O "$t$v\n";
	    $x++;
	} else {
	    print O $line;
	}
    }
    if (!$x) {
	print O "$t$v\n";
    }
    close(I);
    close(O);
    rename "$f.tagtmp", "$f";
}

## getTrackHistory(optional length)
#
# return a list of the most recently played tracks
#
sub getTrackHistory {
    my @list=();
    my $q_directory=$configs{QDIR};
    my $n=$configs{HOW_RECENT};
    local *I;

    $n = 10 if ($n == 0);
    open(I, "/usr/bin/tail -$n $q_directory/history|");
    while (<I>) {
	chomp;
	split / /;
	push @list, $_[0];
    }
    close(I);
    return @list;
}


##############################
#  html rendering functions  #
#                            #
#############################

## printPageHeader(page title, optional http headers, optional html headers)
#
# render http headers and html page top boilerplate
#
sub printPageHeader {
    my $ptitle=$_[0];
    my $cookie=$_[1];
    my $htmlh=$_[2];
    my $line;
    if ($cookie ne "") {
	print $cookie;
    }
    print "Content-type: text/html\n";
    print "\n";
    print "<html>\n";
    print "<META HTTP-EQUIV=\"Pragma\" CONTENT=\"no-cache\">\n";
    if ($htmlh ne "") {
	print $htmlh;
    }
    print "<head>\n";
    print "<title>$configs{BANNER_STRING} $ptitle</title>\n";
    print "</head>\n";
    if ($main::tunebot_user ne "") {
	open(I, "$main::tunebot_root/html/body_head_in.html");
    } else {
	open(I, "$main::tunebot_root/html/body_head_out.html");
    }
    while ($line = <I>) {
	print $line;
    }
    close(I);
    Tunebot::printWhuzzon();
    print "</table><hr><p>\n";
}

## printPageHeaderNoWhuzzon(page title, optional http headers, optional html headers)
#
# render http headers and html page top boilerplate
# this is a special version that never displays the whuzzon line.
#
sub printPageHeaderNoWhuzzon {
    my $ptitle=$_[0];
    my $cookie=$_[1];
    my $htmlh=$_[2];
    my $line;
    if ($cookie ne "") {
	print $cookie;
    }
    print "Content-type: text/html\n";
    print "\n";
    print "<html>\n";
    print "<META HTTP-EQUIV=\"Pragma\" CONTENT=\"no-cache\">\n";
    if ($htmlh ne "") {
	print $htmlh;
    }
    print "<head>\n";
    print "<title>$configs{BANNER_STRING} $ptitle</title>\n";
    print "</head>\n";
    if ($main::tunebot_user ne "") {
	open(I, "$main::tunebot_root/html/body_head_in.html");
    } else {
	open(I, "$main::tunebot_root/html/body_head_out.html");
    }
    while ($line = <I>) {
	print $line;
    }
    close(I);
    print "</table><hr><p>\n";
}

sub printWhuzzon
{
    my $path;
    my $whuz="";
    my @a;
    my $cgibinurl=$configs{"CGI_BIN_URL"};
    my $topurl=$configs{"HTML_TOP_URL"};
    my $q_directory=$configs{"QDIR"};
    my $elaps;
    my $title;
    my $time;

    if ($configs{WHUZZON_IN_HEADER} != 0) {
	$whuz = Tunebot::lookupTagInFile($q_directory . "/nowplaying", "STATUS=");
	if ($whuz ne "") {
	    $whuz .= "<br>";
	}
	$path = Tunebot::lookupTagInFile($q_directory . "/nowplaying", "Q=");
	if ($path ne "") {
	    @a = stat $q_directory . "/nowplaying";
	    $elaps = time() - $a[9];
	    $elaps = sprintf "%d:%02d", $elaps / 60, $elaps % 60;
	    $title = Tunebot::friendlyTrackTitle($path);
	    $title = "<b>$title";
	    $title =~ s/<br>/<\/b> - /;
	    $time = Tunebot::lookupTagInFile($path, "T=");
	    ($time = $path) =~ s/\.mp3$//;
	    $time = $tags{$time, "T"};
	    $time = "?" if ($time eq "");
	    if ($configs{"PLAY_CONTROLS"} != 0) {
		$whuz .= "<a href=$cgibinurl/tuneq.cgi?restart><img src=$topurl/stepback.gif border=0 alt=\"Replay\"></a>\n";
		if (Tunebot::lookupTagInFile("$q_directory/nowplaying", "CTRL")
		eq "pause") {
		    $whuz .= "<a href=$cgibinurl/tuneq.cgi?resume><img src=$topurl/play.gif border=0 alt=\"Play\"></a>\n";
		} else {
		    $whuz .= "<a href=$cgibinurl/tuneq.cgi?pause><img src=$topurl/pause.gif border=0 alt=\"Pause\"></a>\n";
		}
		$whuz .= "<a href=$cgibinurl/tuneq.cgi?kill><img src=$topurl/stepfwd.gif border=0 alt=\"Skip\"></a>\n";
	    }
	    $whuz .= "$title\n";
	    $whuz .= "<a href=\"$cgibinurl/abouttrack.cgi?$path\">$time+$elaps</a>\n";
	}
	print "<tr><td colspan=3 align=center>$whuz</td></tr>\n";
    }
}

## printPageTrailer()
#
# html page bottom boilerplate
#
sub printPageTrailer {
    my $line;
    if ($main::tunebot_user ne "") {
	open(I, $main::tunebot_root . "/html/body_tail_in.html");
    } else {
	open(I, $main::tunebot_root . "/html/body_tail_out.html");
    }
    while ($line = <I>) {
	print $line;
    }
    close(I);
    print "</html>\n";
}

## albumIndexUrl(track_path)
#
# take track path and return URL for album in index.
#
sub albumIndexUrl {
    my ($path) = $_[0];
    $path =~ s/\.mp3$//;
    my ($b) = $path;
    my ($i) = rindex($b, "/");
    if ($i >= 0) {
	$b = substr($b, 0, $i);
    }
    $i = $tags{$b, "N"};
    if ($i ne "") {
	my ($x) = $autotags{$b, "IXA_ABN"};
	if ($x eq "") {
	    return "";
	} else {
	    return "$cgibinurl/dynindex.cgi?abn=$x";
	}
    }
}       

## friendlyTrackTitle(track_path hot_track_name_link)
#
# take track path and render into nice format in html.
#
sub friendlyTrackTitle {
    my ($path) = $_[0];
    my ($hotname) = $_[1];
    $path =~ s/\.mp3$//;
    my ($title) = $tags{$path, "N"};
    if ($title eq "") {
	$title = "<em>" . $path . "</em>";
    }
    if ($hotname != 0) {
	$title = "<a alt=PlayIt href=\"$cgibinurl/tuneput.cgi?" . $path . ".mp3\">" . $title . "</a>";
    }
    my ($art) = $tags{$path, "ARTIST"};
    if ($art ne "") {
	$title = $title . " <em>- " . $art . "</em>";
    }
    my ($b) = $path;
    my ($i) = rindex($b, "/");
    if ($i >= 0) {
	$b = substr($b, 0, $i);
    }
    my ($a) = $b;
    $i = rindex($a, "/");
    if ($i >= 0) {
	$a = substr($a, 0, $i);
    }
    $i = $tags{$a, "N"};
    if ($i eq "") {
	($i = $a) =~ s/.*\///;
    }
    if ($i ne "") {
#	    $title = $title . "<br><em>&nbsp;" . $i;
	my $x = $autotags{$a, "IXA_FAA"};
	if ($x eq "") {
	    $title = $title . "<br><em>&nbsp;" . $i;
	} else {
	    $title="$title<br><em>&nbsp;<a href=\"$cgibinurl/dynindex.cgi?abn=$x\">$i</a>";
	}
	$i = $tags{$b, "N"};
	if ($i eq "") {
	    ($i = $b) =~ s/.*\///;
	}
	if ($i ne "") {
#		$title = $title . " / " . $i;
	    my $x = $autotags{$b, "IXA_ABN"};
	    if ($x eq "") {
		$title = $title . " / " . $i;
	    } else {
		$title="$title / <a href=\"$cgibinurl/dynindex.cgi?abn=$x\">$i</a>";
	    }
	}
	$title = $title . "</em>";
    }
    return $title;
}

## friendlyAlbumTitle(album_path)
#
# take album path and render into nice html format.
#
sub friendlyAlbumTitle {
    my ($title) = "";
    my ($b) = @_;
    my ($a) = $b;
    my ($i) = rindex($a, "/");
    if ($i >= 0) {
	$a = substr($a, 0, $i);
    }
    my $x = $autotags{$a, "IXA_FAA"};
    my $y = $autotags{$b, "IXA_ABN"};
    $i = $tags{$b, "N"};
    if ($i eq "") {
	($i = $b) =~ s/.*\///;
    }
    if ($y eq "") {
	$title = $i;
    } else {
	$title="<a href=\"$cgibinurl/dynindex.cgi?abn=$y\">$i</a>";
    }
    $i = $tags{$a, "N"};
    if ($i eq "") {
	($i = $a) =~ s/.*\///;
    }
    if ($i ne "") {
	if ($x eq "") {
	    $title = $title . " - " . $i;
	} else {
	    $title="$title - <a href=\"$cgibinurl/dynindex.cgi?abn=$x\">$i</a>";
	}
    }
    return $title;
}

#
# index buckets are top-level divisions of the artist index.
#
@ixbktlist=("A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M",
       "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z");
# renderQuickLinks
#
# generate a line of HTML for quick a-z index links
#
sub renderQuickLinks {
    my $quicklinks="<b>\n";
    my $i = "";
    my $c;
    foreach $c (@ixbktlist) {
	if ($autotags{"IXA/I/$c", "IXA_ABN"} ne "") {
	    $quicklinks=$quicklinks.$i."<a href=\"$cgibinurl/dynindex.cgi?abn=".$autotags{"IXA/I/$c", "IXA_ABN"}."\">".$c."</a>\n";
	} else {
	    $quicklinks=$quicklinks.$i.$c."\n";
	}
	$i = "-";
    }
    $quicklinks=$quicklinks."</b>\n";
    return $quicklinks;
}

1;

